{ ######################################################################### }
{ ## FILEIO.PAS: Objekt TFile zur gepufferten und/oder bitweisen Ein-/   ## }
{ ##             Ausgabe aus/in Dateien                                  ## }
{ ######################################################################### }
{ ## Copyright (C) 1999 Martin Fiedler.                                  ## }
{ ## DISCLAIMER: Dieses Programm dient ausschlielich zu Lehr- und Vor-  ## }
{ ##             fhrungszwecken; eine Nutzung in Drittprogrammen,       ## }
{ ##             gewerblich oder nicht-gewerblich, ist ohne vorherige    ## }
{ ##             Absprache mit dem Autoren UNTERSAGT.                    ## }
{ ##             Der Autor ist fr keinerlei Schden, die durch die      ## }
{ ##             Nutzung dieses Programms entstehen, haftbar zu machen.  ## }
{ ##             Die Nutzung dieses Programms erfolgt auf eigenes Risiko.## }
{ ######################################################################### }

unit FileIO;
interface

{ ######################################################################### }
{ ## Compilerunabhngigkeit                                              ## }
{ ######################################################################### }
{$IFDEF FPC} { ----------------------------------------------- FPC-Compiler }
  type BlockIOresult=word;
{$ELSE}{$IFDEF WIN32} { ----------------------------------- Delphi compiler }
  type BlockIOresult=integer;
{$ELSE} { ------------------------------------------- Turbo Pascal compiler }
  type BlockIOresult=word;
{$ENDIF}{$ENDIF} { -------------------------------------------------------- }

{ ######################################################################### }
{ ## Konstanten- und Typendeklarationen                                  ## }
{ ######################################################################### }
const FileBufferSize=1024;  { Gre des Dateipuffers }

{ Konstanten fr die Dateimodi }
const fm_Read=0;
      fm_Write=1;

{ Datentyp eines Dateipuffers }
type FileBuffer=array[0..FileBufferSize-1]of byte;

{ ######################################################################### }
{ ## object TFile                                                        ## }
{ ## Objekt zur Dateibehandlung                                          ## }
{ ######################################################################### }
type PFile=^TFile;
     TFile=object
           { intern verwendete Variablen }            
             Mode:byte;  { Dateimodus (fm_Read oder fm_Write) }
             FVar:file;  { TP-Dateivariable }
             FileEOF:boolean;  { physikalisches Dateiende erreicht? }
             Buffer:^FileBuffer;  { Byte-Puffer }
             BufPos:integer;  { aktuelle Position innerhalb des Puffers }
             BufSize:integer;  { aktuelle Gre des Puffers }
             BitBuffer:longint;  { Zwischenpuffer fr bitweises I/O }
             BitBufSize:byte;  { Gre des Bit-Puffers }

           { dem Programmierer zugngliche Variablen }
             FSize:longint;
               { Dateigre }
             Position:longint;
               { logische Dateiposition }
             EOF:boolean;
               { logisches Dateiende erreicht? }
             function Open(FileName:string;FMode:byte):boolean;
               { Datei ffnen }
             procedure Close;
               { Datei schlieen }
             procedure SeekTo(NewPos:longint);
               { zu einer bestimmten Stelle springen }

             function GetBlock(var Block;Size:word):word;
               { Block lesen (Blockmodus) }
             function GetByte:byte;
               { Byte lesen (Bytemodus) }
             function GetByteAsBits:byte;
               { Byte lesen (Bitmodus) }
             function GetWord:word;
               { Word lesen (Bytemodus) }
             function GetWordAsBits:word;
               { Word lesen (Bitmodus) }
             function GetLong:longint;
               { DWord lesen (Bytemodus) }
             function GetLongAsBits:longint;
               { DWord lesen (Bitmodus) }
             function GetBits(NumBits:byte):word;
               { 1 bis 15 Bits lesen (Bitmodus) }

             procedure PutBlock(var Block;Size:word);
               { Block schreiben (Blockmodus) }
             procedure PutByte(Value:byte);
               { Byte schreiben (Bytemodus) }
             procedure PutByteAsBits(Value:byte);
               { Byte schreiben (Bitmodus) }
             procedure PutWord(Value:word);
               { Word schreiben (Bytemodus) }
             procedure PutWordAsBits(Value:word);
               { Word schreiben (Bitmodus) }
             procedure PutLong(Value:longint);
               { DWord schreiben (Bytemodus) }
             procedure PutLongAsBits(Value:longint);
               { DWord schreiben (Bitmodus) }
             procedure PutBits(Value:word;NumBits:byte);
               { 1 bis 15 Bits schreiben (Bitmodus) }

           { interne Prozeduren }
             procedure InitIO;  { lscht (reinitialisiert) alle Puffer }
             procedure FlushBits;  { Bit-Puffer schreiben & entleeren }
             procedure FlushBuffer;  { Byte-Puffer schreiben & entleeren }
           end;

implementation

{ ######################################################################### }
{ ## procedure TFile.Open                                                ## }
{ ## ffnet eine Datei mit gegebenem Namen im ebenfalls gegebenen Modus  ## }
{ ## und legt den Byte-Puffer dynamisch auf dem Heap an.                 ## }
{ ######################################################################### }
function TFile.Open(FileName:string;FMode:byte):boolean;
var OldFileMode:byte;  { alter Wert von System.FileMode }
begin
  System.Assign(FVar,FileName);  { Dateivariable zuordnen }
  OldFileMode:=System.FileMode;
    { Normalerweise ffnet TP eine Datei mit Reset fr Lese- UND Schreib- }
    { zugriffe, was bei Read-Only-Dateien zwangslufig zu einem I/O-Fehler }
    { fhrt. Um das zu verhindern, wird ber System.FileMode der Lese- oder }
    { Schreibmodus explizit eingestellt (der Wert von FileMode whrend des }
    { Aufrufes von Reset zhlt). Damit TFile nicht mit dem Rest des auf- }
    { rufenden TP-Programms aneinandergert, wird der alte Modus gesichert. }
  System.FileMode:=FMode;  { neuen Dateimodus festlegen }
  {$I-}
  if FMode=fm_Write then System.Rewrite(FVar,1)  { Datei zum Schreiben ... }
                    else System.Reset(FVar,1);   { ... oder Lesen ffnen }
  {$I+}
  if IOResult<>0 then begin Open:=False;exit end
                 else       Open:=True;
  Mode:=FMode;  { Dateimodus in TFile.Mode festhalten }
  System.FileMode:=OldFileMode;  { System.FileMode wiederherstellen }
  System.GetMem(Buffer,FileBufferSize);  { Bytepuffer anlegen }
  if FMode=fm_Read then            { bei Lesezugriffen ... }
    FSize:=System.FileSize(FVar);  { ... zunchst Dateigre ermitteln }
  InitIO;  { Puffer initialisieren }
end;

{ ######################################################################### }
{ ## procedure TFile.Close                                               ## }
{ ## Schliet eine Datei und gibt dessen Byte-Puffer wieder frei.        ## }
{ ######################################################################### }
procedure TFile.Close;
begin
  FlushBuffer;  { zunchst Puffer-Reste schreiben }
  System.Close(FVar);  { Datei schlieen }
  System.FreeMem(Buffer,FileBufferSize);  { Puffer freigeben }
end;

{ ######################################################################### }
{ ## procedure TFile.SeekTo                                              ## }
{ ## Diese Prozedur sollte anstatt System.Seek(TFile.FVar) verwendet     ## }
{ ## werden (die Syntax ist identisch), da andernfalls der logische      ## }
{ ## Dateizeiger TFile.Position nicht angepat wird.                     ## }
{ ######################################################################### }
procedure TFile.SeekTo(NewPos:longint);
begin
  System.Seek(FVar,NewPos);  { zur angegebenen Stelle springen }
  InitIO;  { Puffer re-initialisieren }
end;

{ ######################################################################### }
{ ## function TFile.GetBlock                                             ## }
{ ## Liest einen Block aus der Datei und gibt die Anzahl der wirklich    ## }
{ ## fehlerfrei gelesenen Bytes zurck. (Vorsicht! Nur mit Dateien im    ## }
{ ## Modus fm_Read verwenden, sonst wird's haarig! Dies gilt auch fr    ## }
{ ## smtliche weiteren Get*-Prozeduren!) Sollte nicht nach einer Lese-  ## }
{ ## operation im Byte- oder Bitmodus aufgerufen werden, sonst geraten   ## }
{ ## smtliche Puffer aus dem Sync!                                      ## }
{ ######################################################################### }
function TFile.GetBlock(var Block;Size:word):word;
var ActuallyRead:BlockIOresult;  { Zwischenvariable fr Funktionsergebnis }
begin
  System.BlockRead(FVar,Block,Size,ActuallyRead);  { Block lesen }
  GetBlock:=ActuallyRead;  { Funktionsergebnis setzen }
  inc(Position,ActuallyRead);  { logischen Dateizeiger anpassen }
  FileEOF:=System.EOF(FVar);EOF:=FileEOF;  { EOF-Variablen anpassen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetByte                                             ## }
{ ## Liest ein Byte aus einer Datei. Sollte nicht nach einer Operation   ## }
{ ## im Bitmodus (Get*AsBits, GetBits) verwendet werden.                 ## }
{ ######################################################################### }
function TFile.GetByte:byte;
var ActuallyRead:BlockIOresult;
begin
  if EOF then begin  { Dateiende bereits erreicht? }
    GetByte:=$F6;      { ja, also Standard-"Leer"-Zeichen ausgeben ... }
    exit;              { ... und abbrechen }
  end;

  { Wenn der Puffer schon komplett ausgelesen wurde, mu er wieder gefllt }
  { werden: }
  if BufPos>=BufSize then begin
    System.BlockRead(FVar,Buffer^,FileBufferSize,ActuallyRead);
    BufSize:=ActuallyRead;
      { Puffer wieder fllen }
    BufPos:=0;  { Pufferzeiger auf Anfang setzen }
    if System.EOF(FVar) then FileEOF:=true;
      { physikalisches Dateiende ermitteln }
  end;

  { Byte aus dem Puffer holen und div. Zhler anpassen }
  GetByte:=Buffer^[BufPos];  { Byte lesen }
  inc(BufPos);  { Pufferzeiger weiterschalten }
  inc(Position);  { logischen Dateizeiger weiterschalten }
  if (BufPos>=BufSize) AND FileEOF then EOF:=true;
    { wenn sowohl der Puffer "alle" als auch die Datei zu Ende ist, kann }
    { dies als "EOF" aufgefat werden. }
end;

{ ######################################################################### }
{ ## procedure TFile.GetByteAsBits                                       ## }
{ ## Liest ein Byte aus dem laufenden Bitstrom heraus.                   ## }
{ ######################################################################### }
function TFile.GetByteAsBits:byte;
begin
  GetByteAsBits:=GetBits(8);  { selbsterklrend: 8 Bits, also 1 Byte, lesen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetWord                                             ## }
{ ## Liest ein Word im Bytemodus (intel-Notation / Big-Endian).          ## }
{ ######################################################################### }
function TFile.GetWord:word;
var LoByte,HiByte:byte;  { temporre Variablen fr Low- und High-Byte }
begin
  LoByte:=GetByte;  { Low- und ... }
  HiByte:=GetByte;  { ... High-Byte aus der Datei lesen ... }
  GetWord:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zu einem Word zusammenfgen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetWordAsBits                                       ## }
{ ## Liest ein Word aus dem laufenden Bitstrom.                          ## }
{ ######################################################################### }
function TFile.GetWordAsBits:word;
var LoByte,HiByte:byte;  { temporre Variablen fr Low- und High-Byte }
begin
  LoByte:=GetBits(8);  { Low- und ... }
  HiByte:=GetBits(8);  { ... High-Byte im Bitmodus lesen ... }
  GetWordAsBits:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zu einem Word zusammenfgen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetLong                                             ## }
{ ## Liest ein DWord im Bytemodus (intel-Notation / Big-Endian).         ## }
{ ######################################################################### }
function TFile.GetLong:longint;
var LoByte,HiByte:byte;LoWord,HiWord:word;  { Puffer fr Lo-/Hi-Byte/Word }
begin
  LoByte:=GetByte;  { Low- und ... }
  HiByte:=GetByte;  { ... High-Byte des Low-Words lesen ... }
  LoWord:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zum Low-Word zusammenfgen }
  LoByte:=GetByte;  { Low- und ... }
  HiByte:=GetByte;  { ... High-Byte des High-Words lesen ... }
  HiWord:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zum High-Word zusammenfgen }
  GetLong:=longint(HiWord) SHL 16 OR longint(LoWord);
    { letztendlich Low- und High-Word zum fertigen DWord-Wert zusammenfgen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetLongAsBits                                       ## }
{ ## Liest ein DWord aus dem laufenden Bitsttrom.                        ## }
{ ######################################################################### }
function TFile.GetLongAsBits:longint;
var LoByte,HiByte:byte;LoWord,HiWord:word;  { Puffer fr Lo-/Hi-Byte/Word }
begin
  LoByte:=GetBits(8);  { Low- und ... }
  HiByte:=GetBits(8);  { ... High-Byte des Low-Words lesen ... }
  LoWord:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zum Low-Word zusammenfgen }
  LoByte:=GetBits(8);  { Low- und ... }
  HiByte:=GetBits(8);  { ... High-Byte des High-Words lesen ... }
  HiWord:=word(HiByte) SHL 8 OR word(LoByte);
    { ... und zum High-Word zusammenfgen }
  GetLongAsBits:=longint(HiWord) SHL 16 OR longint(LoWord);
    { letztendlich Low- und High-Word zum fertigen DWord-Wert zusammenfgen }
end;

{ ######################################################################### }
{ ## procedure TFile.GetBits                                             ## }
{ ## Liest Einzelbits aus dem Bitstrom.                                  ## }
{ ######################################################################### }
function TFile.GetBits(NumBits:byte):word;
begin
  { solange nicht gengend Bits im Puffer sind, mssen welche nachgeladen }
  { werden: }
  while BitBufSize<NumBits do begin
    BitBuffer:=BitBuffer SHL 8;  { Platz im Bit-Puffer schaffen }
    inc(BitBuffer,GetByte);  { ein Byte in die Leerstelle lesen }
    inc(BitBufSize,8);  { Bitpuffergre anpassen }
  end;
  GetBits:=BitBuffer SHR (BitBufSize-NumBits) AND ((1 SHL NumBits)-1);
    { gegebene Anzahl von Bits aus dem Bit-Puffer extrahieren }
  dec(BitBufSize,NumBits);  { Bitpuffergre anpassen }
end;

{ ######################################################################### }
{ ## procedure TFile.PutBlock                                            ## }
{ ## Schreibt einen Block in die Datei. (Vorsicht! Nur mit Dateien im    ## }
{ ## Modus fm_Write verwenden, sonst wird's haarig! Dies gilt auch fr   ## }
{ ## smtliche weiteren Put*-Prozeduren!) Sollte nicht nach einer        ## }
{ ## Schreiboperation im Byte- oder Bitmodus aufgerufen werden, sonst    ## }
{ ## geraten smtliche Puffer aus dem Sync!                              ## }
{ ######################################################################### }
procedure TFile.PutBlock(var Block;Size:word);
var ActuallyWritten:BlockIOresult;
      { um Fehler abzufangen, wird das Ergebnis von System.BlockWrite }
      { bercksichtigt }
begin
  System.BlockWrite(FVar,Block,Size,ActuallyWritten);  { Block schreiben }
  inc(Position,ActuallyWritten);  { logischen Dateizeiger anpassen }
end;

{ ######################################################################### }
{ ## procedure TFile.PutByte                                             ## }
{ ## Gibt ein Byte in die Datei aus. Sollte nicht nach Bit-Operationen   ## }
{ ## verwendet werden.                                                   ## }
{ ######################################################################### }
procedure TFile.PutByte(Value:byte);
var Dummy:BlockIOresult;  { zum Abfangen von Fehlern }
begin
  Buffer^[BufPos]:=Value;  { Wert im Puffer absetzen }
  inc(BufPos);  { Pufferzeiger weiterschalten }
  inc(Position);  { logischen Dateizeiger weiterschalten }
  { wenn der Puffer voll ist, mu er geleert werden: }
  if BufPos>=FileBufferSize then begin
    System.BlockWrite(FVar,Buffer^,FileBufferSize,Dummy);
      { Pufferinhalt ausgeben }
    BufPos:=0;  { Pufferzeiger auf Pufferanfang setzen }
  end;
end;

{ ######################################################################### }
{ ## procedure TFile.PutByteAsBits                                       ## }
{ ## Gibt ein Byte in den laufenden Bitstrom aus.                        ## }
{ ######################################################################### }
procedure TFile.PutByteAsBits(Value:byte);
begin
  PutBits(Value,8);
end;

{ ######################################################################### }
{ ## procedure TFile.PutWord                                             ## }
{ ## Gibt ein Word in intel-Notation (Big-Endian) im Bytemodus aus.      ## }
{ ######################################################################### }
procedure TFile.PutWord(Value:word);
begin
  PutByte(Lo(Value));  { Low-Byte ... }
  PutByte(Hi(Value));  { ... und High-Byte ausgeben }
end;

{ ######################################################################### }
{ ## procedure TFile.PutWordAsBits                                       ## }
{ ## Gibt ein Word in den laufenden Bitstrom aus.                        ## }
{ ######################################################################### }
procedure TFile.PutWordAsBits(Value:word);
begin
  PutBits(Lo(Value),8);  { Low-Byte ... }
  PutBits(Hi(Value),8);  { ... und High-Byte ausgeben }
end;

{ ######################################################################### }
{ ## procedure TFile.PutLong                                             ## }
{ ## Gibt ein DWord in intel-Notation (Big-Endian) im Bytemodus aus.     ## }
{ ######################################################################### }
procedure TFile.PutLong(Value:longint);
var LoWord,HiWord:word;  { temporre Variablen fr Low- und High-Word }
begin
  LoWord:=longint(Value AND $FFFF);  { Low-Word extrahieren }
  HiWord:=longint(Value SHR 16);     { High-Word extrahieren }
  PutByte(Lo(LoWord));  { Lo-Byte des Lo-Words schreiben }
  PutByte(Hi(LoWord));  { Hi-Byte des Lo-Words schreiben }
  PutByte(Lo(HiWord));  { Lo-Byte des Hi-Words schreiben }
  PutByte(Hi(HiWord));  { Hi-Byte des Hi-Words schreiben }
end;

{ ######################################################################### }
{ ## procedure TFile.PutLongAsBits                                       ## }
{ ## Gibt ein DWord in den laufenden Bitstrom aus.                       ## }
{ ######################################################################### }
procedure TFile.PutLongAsBits(Value:longint);
var LoWord,HiWord:word;  { temporre Variablen fr Low- und High-Word }
begin
  LoWord:=longint(Value AND $FFFF);  { Low-Word extrahieren }
  HiWord:=longint(Value SHR 16);     { High-Word extrahieren }
  PutBits(Lo(LoWord),8);  { Lo-Byte des Lo-Words schreiben }
  PutBits(Hi(LoWord),8);  { Hi-Byte des Lo-Words schreiben }
  PutBits(Lo(HiWord),8);  { Lo-Byte des Hi-Words schreiben }
  PutBits(Hi(HiWord),8);  { Hi-Byte des Hi-Words schreiben }
end;

{ ######################################################################### }
{ ## procedure TFile.PutBits                                             ## }
{ ## Gibt eine beliebige Anzahl von Bits (1 bis 15) in den Bitstrom aus. ## }
{ ######################################################################### }
procedure TFile.PutBits(Value:word;NumBits:byte);
begin
  BitBuffer:=BitBuffer SHL NumBits OR Value;
    { Wert in den Bit-Puffer einschieben }
  inc(BitBufSize,NumBits);
    { Gre des Bit-Puffers anpassen }
  { wenn der Puffer bereits ein Byte oder mehr umfat, kann (ja, mu sogar) }
  { es gleich ausgegeben werden: }
  while BitBufSize>=8 do begin
    PutByte(BitBuffer SHR (BitBufSize-8) AND $FF);
      { oberste 8 Bit des Bit-Puffers ausgeben }
    dec(BitBufSize,8);  { Puffergre anpassen }
  end;
end;

{ ######################################################################### }
{ ## procedure TFile.InitIO                                              ## }
{ ## Re-initialisiert smtliche Pufferzeiger. Nur zur internen Verwen-   ## }
{ ## dung.                                                               ## }
{ ######################################################################### }
procedure TFile.InitIO;
begin
  Position:=System.FilePos(FVar);  { Dateiposition ermitteln }
  EOF:=false;FileEOF:=false;  { Dateiende-Marken lschen }
  BufSize:=0;BufPos:=0;  { Byte-Puffer lschen }
  BitBuffer:=0;BitBufSize:=0;  { Bit-Puffer lschen }
end;

{ ######################################################################### }
{ ## procedure TFile.FlushBits                                           ## }
{ ## Gibt evtl. noch im Bit-Puffer befindliche Bits aus. Nur zur         ## }
{ ## internen Verwendung.                                                ## }
{ ######################################################################### }
procedure TFile.FlushBits;
begin
  if BitBufSize>0 then                { Wenn noch etwas im Puffer steht ... }
    PutByte(BitBuffer SHL (8-BitBufSize));     { ... dessen Inhalt ausgeben }
  BitBufSize:=0;  { Puffer ber den Grenwert lschen }
end;

{ ######################################################################### }
{ ## procedure TFile.FlushBuffer                                         ## }
{ ## Gibt sowohl die Bits im Bit-Puffer als auch die restlichen Bytes im ## }
{ ## Byte-Puffer aus. Nur zur internen Verwendung.                       ## }
{ ######################################################################### }
procedure TFile.FlushBuffer;
var ActuallyWritten:BlockIOresult;  { zum Abfangen von I/O-Fehlern }
begin
  if Mode=fm_Read then exit;  { geht natrlich nicht im Lesemodus }
  FlushBits;  { zuerst Bit-Puffer entleeren }
  { Wenn noch etwas im Byte-Puffer steht, diesen entleeren: }
  if BufPos>0 then begin
    System.BlockWrite(FVar,Buffer^,BufPos,ActuallyWritten);
      { Pufferinhalt schreiben }
    inc(Position,ActuallyWritten);
      { logischen Dateizeiger anpassen }
  end;
end;

end.

