{unit MJBase;}

interface
uses Windows, ShellAPI, SysUtils, Classes, Graphics, inifiles, FileIO, JPEG;


procedure varint;
procedure InitSprites;
procedure Flip;
procedure Redraw;
procedure QuitProgram;
procedure SaveConfig;
procedure LoadConfig;

procedure AddTile(_x,_y:integer;_plane,_sprite:byte);
procedure KillTile(Num:integer);
procedure NewGame;
procedure MoveCheck;
function FindTile(px,py:integer):integer;
function TilePos(Num:integer):TRect;
procedure Enlightenment(Area:TRect);
procedure UseTile(what:word);
procedure TimerHandler;
procedure Hint;
procedure Undo;
procedure Redo;
procedure SaveGame;
procedure LoadGame;
procedure PauseGame;
procedure ContinueGame;
procedure PWrite(UseColor:TColor;const Text:string);
procedure RunHelp;

const TileSprites=42;  { Anzahl der Sprites }
      TileWidth=38;  { Tile-Breite }
      TileHeight=54;  { Tile-Hhe }
      FieldOffsetX=35;  { Startposition des Spielfeldes (X) }
      FieldOffsetY=20;  { Startposition des Spielfeldes (Y) }
      HeightOffset=9;  { Hhenoffset zwischen einzelnen Ebenen }
      EnlightenmentRate=32;  { Helligkeitsoffset der Hervorhebung }
      HintPenalty=10000;  { Strafzeit bei Hinweis }
      WLFontSize=54;  { Gre des "Gewonnen"/"Verloren"-Textes }
      WLTShadowOfs=4;  { Offset des "Gew."/"Verl."-Schattens }
      AlreadyFreed:boolean=False;  { wurde QuitProgram schon aufgerufen? }
      CleverlyGenerated:boolean=false;  { wurde dieses Spiel lsbar erzeugt? }
      PixForm:TPixelFormat=pfDevice;  { Pixelformat }
      ShowRenderSpeed:boolean=False;  { Rendergeschwindigkeit anzeigen? }
      AvoidUnsolveableGames:boolean=true;  { Unlsbare Spiele verhindern? }
      PlayerName:string='MJ2000';  { Spielername }
      SGFileName:string='QuickSave.mj2k';
      IniFileName='.\MJ2000.ini';
      ForceRepaint:boolean=false;  { bei jedem Neuzeichnen TImage.Repaint aufrufen? }

type TTile=record
             Active:boolean;
             x,y:integer;
             Plane:byte;
             Sprite:byte;
             Moveable:boolean;
           end;

var SBack:TBitmap;
    STiles:TBitmap;
    SFrame:TBitmap;
    Buf:TBitmap;
    SGFile:TFile;
    cfgFile:TIniFile;

    BackXSize,BackYSize:integer;

    SpritesX,SpritesY:array[0..64]of integer;  { Positionen der Sprites }
    NullSprite:TRect;  { Position des Null-Sprites }

    Tiles:array[byte]of TTile;
    TilesLeft:array[1..64]of byte; { Anzahl der verfgbaren Tiles pro Sprite }

    MaxTiles:integer;  { Nummer des grten derzeit aktiven Tiles }
    TotalTiles:integer;  { Anzahl der Tiles insgesamt }
    MaxPlanes:byte;  { Nummer der hchsten verwendeten Ebene }
    MaxTileX,MaxTileY:integer;  { grte Tile-X/Y-Koordinate }
    TileFitMin,TileFitMax:array[1..64]of byte;
      { welche Spritenummern lassen sich mit welchen Sprites wegklicken? }

    Marked:boolean;  { Ist derzeit ein Tile markiert? }
    MarkTile:integer;  { Nummer des markierten Tiles }
    Changed:boolean;  { globales Flag: alles Neuzeichnen und -berechnen? }

    Active:array[1..146]of integer;MaxActive:integer;
      { Liste aller derzeit beweglichen Tiles }
    Moves:integer;  { Anzahl der derzeit mglichen Zge }

    History:array[1..80,0..2]of integer;  { Undo-Liste }
    HistMax:integer;  { Spitze der Undo-Liste (maximale Eintraxnummer) }
    HistPos:integer;  { Position in der Undo-Liste (letzter Zug) }

    Stuck:boolean;  { Ist das Spiel zum Stehen gekommen? }
    Won:boolean;  { Wurde das Spiel gewonnen? }
    Game:boolean;  { Ist das Spiel aktiv? }
    Undone:boolean;  { konnte ein Undo ausgefhrt werden? }

    GaveHint:boolean;  { Wurde ein Hint gegeben? }
    HintTile1,HintTile2:integer;  { beim Hint markierte Teile }

    Time_NextRefresh:longint;  { nxter Uhr-Refresh frhestens ... }
    Time_ActiveSince:longint;  { Uhr ist aktiv seit ... }
    Time_Offset:longint;  { Uhr startete zur Delta-Zeit ... }
    Time_Delta:longint;  { aktuelle Delta-Zeit (=Spielzeit in Ticks) }
    RenderSpeed:longint;  { letzte Rendering-Zeit }

    WonTextX,LoseTextX,WLTextY:integer;  { Positionen der Gewinn-/Verliertexte }
    WonBuildMode:boolean;  { Richtung der Gewinnanimation }

    PPenY:integer;  { Stiftposition im Pause-Men }


implementation
uses MJMain,MJScore;

procedure CreateSpriteList;
var CSprite:byte;x:integer;
  procedure AddSprite(_x,_y:integer);
  begin
    SpritesX[CSprite]:=_x;
    SpritesY[CSprite]:=_y;
    inc(CSprite);
  end;
begin
  CSprite:=0;
  AddSprite(TileWidth*7,TileHeight*4);
  for x:=0 to 8 do AddSprite(x*TileWidth,TileHeight);
  for x:=0 to 8 do AddSprite(x*TileWidth,0);
  for x:=0 to 8 do AddSprite(x*TileWidth,TileHeight*2);
  for x:=0 to 3 do AddSprite(x*TileWidth+TileWidth*4,TileHeight*3);
  for x:=0 to 2 do AddSprite(x*TileWidth+TileWidth*4,TileHeight*4);
  for x:=0 to 3 do AddSprite(x*TileWidth,TileHeight*3);
  for x:=0 to 3 do AddSprite(x*TileWidth,TileHeight*4);
  NullSprite:=
    Rect(SpritesX[0],SpritesY[0],SpritesX[0]+TileWidth,SpritesY[0]+TileHeight);
end;

procedure InitSprites;
begin
  SBack.LoadFromFile('MJBack.bmp');
  SBack.PixelFormat:=PixForm;
  BackXSize:=SBack.Width;BackYSize:=SBack.Height;
  STiles.LoadFromFile('MJTiles.bmp');
  STiles.PixelFormat:=PixForm;
  SFrame.LoadFromFile('MJTFrame.bmp');
  SFrame.PixelFormat:=PixForm;
  SFrame.TransparentColor:=0;
  SFrame.TransparentMode:=tmFixed;
  SFrame.Transparent:=True;
  Buf.Width:=640;Buf.Height:=480;
  Buf.PixelFormat:=PixForm;
  with Buf.Canvas do begin
    Brush.Style:=bsClear;
    Font.Name:='Trebuchet MS';
    Font.Style:=[fsBold];
    Font.Size:=WLFontSize;
    WonTextX:=(640-TextWidth('GEWONNEN!')) SHR 1;
    LoseTextX:=(640-TextWidth('VERLOREN!')) SHR 1;
    WLTextY:=(480-TextHeight('X')-WLFontSize) SHR 1;
  end;
end;

procedure varint;
begin randomize;
  LoadConfig;
  with MainForm do begin
    MNoUnsolveableGames.Checked:=AvoidUnsolveableGames;
    case PixForm of
      pfDevice:MFDevice.Checked:=True;
      pf32bit:MF32.Checked:=True;
      pf24bit:MF24.Checked:=True;
      pf16bit:MF16.Checked:=True;
      pf15bit:MF15.Checked:=True;
      pf8bit:MF8.Checked:=True;
      pf4bit:MF4.Checked:=True;
    else
      PixForm:=pfDevice;
      MFDevice.Checked:=True;
    end;
    MShowSpeed.Checked:=ShowRenderSpeed;
    MForceRepaint.Checked:=ForceRepaint;
  end;

  SBack:=TBitmap.Create;
  STiles:=TBitmap.Create;
  SFrame:=TBitmap.Create;
  Buf:=TBitmap.Create;

  InitSprites;
  CreateSpriteList;
  NewGame;
  MoveCheck;
  Redraw;
end;

procedure Flip;
begin
  MainForm.Field.Canvas.Draw(0,0,Buf);
end;

procedure Redraw;
var x,y:integer;CPlane:byte;Num,OfsX,OfsY:integer;TileRect:TRect;
    TStart,TEnd:integer;
begin
  TStart:=GetTickCount;
  y:=0;
  while y<480 do begin
    x:=0;
    while x<640 do begin
      Buf.Canvas.Draw(x,y,SBack);
      inc(x,BackXSize);
    end;
    inc(y,BackYSize);
  end;
  for CPlane:=1 to MaxPlanes do begin
    OfsX:=FieldOffsetX+(CPlane-1)*HeightOffset;
    OfsY:=FieldOffsetY-(CPlane-1)*HeightOffset;
    for Num:=1 to MaxTiles do
      with Tiles[Num] do
        if Active AND (Plane=CPlane) then
          Buf.Canvas.Draw(x+OfsX-HeightOffset,y+OfsY-1,SFrame);
    for Num:=1 to MaxTiles do
      with Tiles[Num] do
        if Active AND (Plane=CPlane) then begin
          TileRect:=Rect(x+OfsX,y+OfsY,x+OfsX+TileWidth,y+OfsY+TileHeight);
          Buf.Canvas.CopyRect(TileRect,STiles.Canvas,
            Rect(SpritesX[Sprite],SpritesY[Sprite],
                 SpritesX[Sprite]+TileWidth,SpritesY[Sprite]+TileHeight));
          if Marked AND (MarkTile=Num) then Enlightenment(TileRect);
        end;
  end;
  TEnd:=GetTickCount;
  RenderSpeed:=TEnd-TStart;

  with MainForm.Status do
    if Stuck then begin
      if Won then begin
        Panels[0].Text:='GEWONNEN!';
        with Buf.Canvas do begin
          Font.Color:=$00000000;
          TextOut(WonTextX+WLTShadowOfs,WLTextY+WLTShadowOfs,'GEWONNEN!');
          Font.Color:=$0000FF00;
          TextOut(WonTextX,WLTextY,'GEWONNEN!');
        end;
      end else begin
        Panels[0].Text:='KEINE WEITEREN ZGE!';
        with Buf.Canvas do begin
          Font.Color:=$00000000;
          TextOut(LoseTextX+WLTShadowOfs,WLTextY+WLTShadowOfs,'VERLOREN!');
          Font.Color:=$000000FF;
          TextOut(LoseTextX,WLTextY,'VERLOREN!');
        end;
      end;
    end else begin
      Panels[0].Text:='mgliche Zge: '+IntToStr(Moves);
    end;

  Flip;
  GaveHint:=False;
end;

procedure QuitProgram;
begin
  if AlreadyFreed then exit;
  SaveConfig;
  Buf.Free;
  SFrame.Free;
  STiles.Free;
  SBack.Free;
  AlreadyFreed:=True;
end;


procedure AddTile(_x,_y:integer;_plane,_sprite:byte);
begin
  inc(MaxTiles);
  with Tiles[MaxTiles] do begin
    Active:=True;
    x:=_x;y:=_y;Plane:=_Plane;Sprite:=_Sprite;
    Moveable:=False;
  end;
  if _Plane>MaxPlanes then MaxPlanes:=_Plane;
  if _x>MaxTileX then MaxTileX:=_x;
  if _y>MaxTileY then MaxTileY:=_y;
end;

procedure KillTile(Num:integer);  { Lscht ein Tile }
begin
  Tiles[Num].Active:=False;
  while not Tiles[MaxTiles].Active do dec(MaxTiles);
end;

function FindTile(px,py:integer):integer;
var i:integer;BestPlane,BestTile:integer;tx,ty:integer;
begin
  BestPlane:=0;BestTile:=0;
  for i:=1 to MaxTiles do
    with Tiles[i] do begin
      tx:=FieldOffsetX+x+(Plane-1)*HeightOffset;
      ty:=FieldOffsetY+y-(Plane-1)*HeightOffset;
      if Active AND (tx<=px) AND (ty<=py) AND (tx+TileWidth>px)
      AND (ty+TileHeight>py) AND (Plane>BestPlane) then begin
        BestPlane:=Plane;
        BestTile:=i;
      end;
    end;
  FindTile:=BestTile;
end;

function TilePos(Num:integer):TRect;
begin
  with Tiles[Num] do with Result do begin
    Left:=FieldOffsetX+x+(Plane-1)*HeightOffset;
    Top:=FieldOffsetY+y-(Plane-1)*HeightOffset;
    Right:=Left+TileWidth;
    Bottom:=Top+TileHeight;
  end;
end;

procedure Enlightenment(Area:TRect);
var x,y,i,StartAdr,EndAdr:integer;Line:PByteArray;
begin
  case PixForm of
    pf32bit:begin
              for y:=Area.Top to Area.Bottom do begin
                StartAdr:=Area.Left*4;
                EndAdr:=Area.Right*4-1;
                Line:=Buf.ScanLine[y];
                for x:=StartAdr to EndAdr do begin
                  i:=Line[x]+EnlightenmentRate;
                  if i>255 then i:=255;
                  Line[x]:=i;
                end;
              end;
            end;
    pf24bit:begin
              for y:=Area.Top to Area.Bottom do begin
                StartAdr:=Area.Left*3;
                EndAdr:=Area.Right*3-1;
                Line:=Buf.ScanLine[y];
                for x:=StartAdr to EndAdr do begin
                  i:=Line[x]+EnlightenmentRate;
                  if i>255 then i:=255;
                  Line[x]:=i;
                end;
              end;
            end;
    pf16bit,pf15bit:begin
              for y:=Area.Top to Area.Bottom do begin
                StartAdr:=Area.Left*2;
                EndAdr:=Area.Right*2-1;
                Line:=Buf.ScanLine[y];
                for x:=StartAdr to EndAdr do
                  Line[x]:=Line[x] XOR $FF;
              end;
            end;
    pf8bit:begin
             for y:=Area.Top to Area.Bottom do begin
               StartAdr:=Area.Left;
               EndAdr:=Area.Right-1;
               Line:=Buf.ScanLine[y];
               for x:=StartAdr to EndAdr do
                 Line[x]:=Line[x] XOR $FF;
             end;
           end;
    pf4bit:begin
             for y:=Area.Top to Area.Bottom do begin
               StartAdr:=Area.Left SHR 1;
               EndAdr:=(Area.Right-1) SHR 1;
               Line:=Buf.ScanLine[y];
               for x:=StartAdr to EndAdr do
                 Line[x]:=Line[x] XOR $FF;
             end;
           end;
    pfDevice:with Buf.Canvas do begin
               CopyMode:=cmMergePaint;
               CopyRect(Area,STiles.Canvas,NullSprite);
               CopyMode:=cmSrcCopy;
             end;
  end;
end;

procedure TimerHandler;
var TimeSec,Min,Sec:integer;SecStr:string[2];s:string;
begin
  if not Game then Exit;
  if Won then begin
    if WonBuildMode then Undo else Redo;
    if not Undone then WonBuildMode:=not(WonBuildMode);
  end;
  if not Stuck then Time_Delta:=Timer-Time_ActiveSince+Time_Offset;
  if Stuck AND not(ShowRenderSpeed) then exit;
  if Timer<Time_NextRefresh then exit;
  Time_NextRefresh:=Timer+200;
  TimeSec:=Time_Delta DIV 1000;
  Min:=TimeSec DIV 60;
  Sec:=TimeSec MOD 60;
  SecStr:=char(Sec DIV 10+48)+char(Sec MOD 10+48);
  s:='Zeit: '+IntToStr(Min)+':'+SecStr;
  if ShowRenderSpeed then
    s:=s+'  ['+IntToStr(RenderSpeed)+' ms]';
  MainForm.Status.Panels[1].Text:=s;
end;

procedure Undo;  { Undo-Funktion }
var i:integer;
begin
  if HistPos<1 then begin
    Undone:=False;
    exit;
  end else
    Undone:=True;
  if not Won then begin
    Time_Offset:=History[HistPos,0];
    Time_ActiveSince:=Timer;
  end;
  i:=History[HistPos,1];
    Tiles[i].Active:=True;
    if MaxTiles<i then MaxTiles:=i;
  i:=History[HistPos,2];
    Tiles[i].Active:=True;
    if MaxTiles<i then MaxTiles:=i;
  dec(HistPos);
  Marked:=False;
  if not Won then MoveCheck;
  Redraw;
end;

procedure Redo;  { Redo-Funktion }
begin
  if HistPos>=HistMax then begin
    Undone:=False;
    exit;
  end else
    Undone:=True;
  inc(HistPos);
  if not Won then begin
    Time_Offset:=History[HistPos,0];
    Time_ActiveSince:=Timer;
  end;
  KillTile(History[HistPos,1]);
  KillTile(History[HistPos,2]);
  Marked:=False;
  if not Won then MoveCheck;
  Redraw;
end;

procedure SaveGame;
var i:integer;
begin
  SGFile.Open(SGFileName,fm_Write);
  SGFile.PutLong($4B324A4D);
  SGFile.PutByte(TotalTiles);
  if TotalTiles>0 then begin
    SGFile.PutByte(MaxTiles);
    for i:=1 to TotalTiles do
      with Tiles[i] do begin
        SGFile.PutBits(byte(Active),1);
        SGFile.PutBits(x,10);
        SGFile.PutBits(y,10);
        SGFile.PutBits(Plane,4);
        SGFile.PutBits(Sprite,6);
        SGFile.PutBits(byte(Moveable),1);
      end;
  end;
  SGFile.PutByteAsBits(HistMax);
  if HistMax>0 then begin
    SGFile.PutByteAsBits(HistPos);
    for i:=1 to HistMax do begin
      SGFile.PutByteAsBits(History[i,1]);
      SGFile.PutByteAsBits(History[i,2]);
      SGFile.PutWordAsBits(integer(History[i,0] SHR 6));
    end;
  end;
  SGFile.PutWordAsBits(integer(Time_Delta SHR 6));
  SGFile.Close;
end;

procedure LoadGame;
var i:integer;
begin
  SGFile.Open(SGFileName,fm_Read);
  i:=SGFile.GetLong;
  if i<>$4B324A4D then begin
    SGFile.Close;
  exit;end;
  TotalTiles:=SGFile.GetByte;
  if TotalTiles>0 then begin
    MaxTiles:=SGFile.GetByte;
    for i:=1 to TotalTiles do
      with Tiles[i] do begin
        Active:=boolean(SGFile.GetBits(1));
        x:=SGFile.GetBits(10);
        y:=SGFile.GetBits(10);
        Plane:=SGFile.GetBits(4);
        Sprite:=SGFile.GetBits(6);
        Moveable:=boolean(SGFile.GetBits(1));
      end;
  end else MaxTiles:=0;
  HistMax:=SGFile.GetByteAsBits;
  if HistMax>0 then begin
    HistPos:=SGFile.GetByteAsBits;
    for i:=1 to HistMax do begin
      History[i,1]:=SGFile.GetByteAsBits;
      History[i,2]:=SGFile.GetByteAsBits;
      History[i,0]:=SGFile.GetWordAsBits SHL 6;
    end;
  end else HistPos:=0;
  Time_Offset:=SGFile.GetWordAsBits SHL 6;
  Time_ActiveSince:=Timer;
  SGFile.Close;
  Changed:=True;Stuck:=False;Won:=False;Game:=True;
  MoveCheck;Redraw;
end;

procedure PauseGame;
var LogoImage:TJPEGImage;
begin
  Game:=False;
  LogoImage:=TJPEGImage.Create;
  LogoImage.LoadFromFile('MJ2KLogo.jpg');
  Buf.Canvas.Draw(0,0,LogoImage);
  LogoImage.Free;
  Flip;
  PPenY:=460;
end;

procedure ContinueGame;
begin
  Game:=True;
  Time_Offset:=Time_Delta;Time_ActiveSince:=Timer;
  Buf.PixelFormat:=PixForm;
  Redraw;Flip;
end;

procedure PWrite(UseColor:TColor;const Text:string);
begin
  with MainForm.Field.Canvas do begin
    Font.Name:='Tahoma';
    Font.Style:=[];
    Font.Size:=8;
    Brush.Style:=bsClear;
    Font.Color:=0;
    TextOut(11,PPenY-1,Text);
    TextOut(10,PPenY-1,Text);
    TextOut(9,PPenY-1,Text);
    TextOut(9,PPenY,Text);
    TextOut(9,PPenY+1,Text);
    TextOut(10,PPenY+1,Text);
    TextOut(11,PPenY+1,Text);
    TextOut(11,PPenY,Text);
    Font.Color:=UseColor;
    TextOut(10,PPenY,Text);
  end;
  dec(PPenY,14);
end;

const hex:string[16]='0123456789ABCDEF';

procedure SaveConfig;
var PlayerNo,b:integer; s:string[63];
    t:System.Text;
procedure AddByte(b:byte);begin s:=s+hex[1+(b SHR 4)]+hex[1+(b AND 15)]; end;
procedure AddWord(w:word);begin AddByte(w SHR 8);AddByte(w AND 255); end;
procedure AddLong(l:cardinal);begin AddWord(l SHR 16);AddWord(l AND $FFFF); end;
begin
  AssignFile(t,IniFileName); ReWrite(t);
  writeln(t,'[MJ2000]');
  writeln(t,'AvoidUnsolveableGames=',byte(AvoidUnsolveableGames));
  writeln(t,'PlayerName=',PlayerName);
  case PixForm of
    pf32bit:writeln(t,'PixelFormat=32');
    pf24bit:writeln(t,'PixelFormat=24');
    pf16bit:writeln(t,'PixelFormat=16');
    pf15bit:writeln(t,'PixelFormat=15');
    pf8bit:writeln(t,'PixelFormat=8');
    pf4bit:writeln(t,'PixelFormat=4');
  else
    writeln(t,'PixelFormat=0');
  end;
  writeln(t,'ShowRenderSpeed=',byte(ShowRenderSpeed));
  writeln(t,'ForceRepaint=',byte(ForceRepaint));
  writeln(t);
  writeln(t,'[Highscores]');
  for PlayerNo:=1 to 10 do begin
    s[0]:=#0;
    with Scores[PlayerNo] do begin
      for b:=0 to length(Name) do
        AddByte(byte(Name[b]));
      AddWord(Time DIV 1000);
      AddLong(trunc(Date));
    end;
    writeln(t,PlayerNo,'=',s);
  end;
  CloseFile(t);
end;

procedure LoadConfig;
var PlayerNo:integer; s:string[63]; b,m:byte; c:^char;
function GetByte:byte;var a,b:char;begin a:=upcase(c^);inc(c); b:=upcase(c^);inc(c);
  GetByte:=(pos(a,hex)-1) SHL 4 OR (pos(b,hex)-1);end;
function GetWord:word;begin GetWord:=GetByte SHL 8 OR GetByte;end;
function GetLong:cardinal;begin GetLong:=GetWord SHL 16 OR GetWord;end;
begin
  cfgFile:=TIniFile.Create(IniFileName);
  AvoidUnsolveableGames:=cfgFile.ReadBool('MJ2000','AvoidUnsolveableGames',AvoidUnsolveableGames);
  PlayerName:=cfgFile.ReadString('MJ2000','PlayerName',PlayerName);
  case cfgFile.ReadInteger('MJ2000','PixelFormat',0) of
    32:PixForm:=pf32bit;
    24:PixForm:=pf24bit;
    16:PixForm:=pf16bit;
    15:PixForm:=pf15bit;
    8:PixForm:=pf8bit;
    4:PixForm:=pf4bit;
  else
    PixForm:=pfDevice;
  end;
  ShowRenderSpeed:=cfgFile.ReadBool('MJ2000','ShowRenderSpeed',ShowRenderSpeed);
  ForceRepaint:=cfgFile.ReadBool('MJ2000','ForceRepaint',ForceRepaint);
  for PlayerNo:=1 to 10 do begin
    FillChar(s,sizeof(s),0);
    s:=cfgFile.ReadString('Highscores',IntToStr(PlayerNo),'nscht');
    if s<>'nscht' then begin
      c:=@s[1];
      with Scores[PlayerNo] do begin
        m:=GetByte;
        if (m>0) AND (m<=20) then begin
          Name[0]:=char(m);
          for b:=1 to m do
            Name[b]:=char(GetByte);
          Time:=GetWord*1000;
          Date:=GetLong+0.5;
        end;
      end;
    end;
  end;
  cfgFile.Free;
end;

procedure RunHelp;
var res:integer;
begin
  res:=ShellExecute(MainForm.Handle,'open','MJ2000.htm','','',SW_SHOW);
  if res<32 then
    MessageBox(MainForm.Handle,'Kann Hilfe nicht aufrufen','MJ2000-Fehler',
               MB_OK+MB_ICONWARNING);
end;

{end.}
